﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using MindFusion.Scheduling;
using MindFusion.Diagramming;
using System.Data.Entity.Validation;
using System.Data.Entity.Infrastructure;
using System.Diagnostics;

namespace TicketBookingSystem
{
    public partial class Form1 : Form
    {
        EventsTicketsEntities records;
        double basePrice;

        public Form1()
        {
            InitializeComponent();
            records = new EventsTicketsEntities();

            diagram.ClearAll();
            CreateSection(
                0, new int[] { 12, 12, 13, 13, 14, 14, 15, 15 },
                10, 10, 12, 1, 4, 200);
            diagram.ResizeToFitItems(5);
            
            CreateSection(
                8, new int[] { 14, 14, 14, 14, 14, 14, 14, 14 },
                diagram.Bounds.Bottom, 10, 12, 1, 4, 0);
            diagram.ResizeToFitItems(5);
            diagramView.ZoomToFit();
            generateOccupiedNode();

            calendar.TooltipDisplaying += Calendar_TooltipDisplaying;
            calendar.ShowToolTips = true;
            //calendar.ItemSelection

            calendar.AllowInplaceEdit = false;
            calendar.ItemSettings.Size = 10;            
            calendar.ItemSettings.ShowClocks = MindFusion.Scheduling.WinForms.ShowClocks.Never;
            calendar.ItemSettings.Style.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(102, 154, 204));
            calendar.ItemSettings.Style.BorderBottomColor = calendar.ItemSettings.Style.BorderLeftColor =
                calendar.ItemSettings.Style.BorderRightColor = calendar.ItemSettings.Style.BorderTopColor =
                Color.FromArgb(192, 192, 192);
            calendar.ItemSettings.SelectedItemStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(206, 0, 0));
            calendar.Selection.SelectedElementsStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(224, 233, 233));

            loadEvents();
        }

        private void generateOccupiedNode()
        {
            float x = diagram.Bounds.Left + diagram.Bounds.Width / 2 - 50;
            float y = diagram.Bounds.Top + diagram.Bounds.Height / 2 - 30;

            RectangleF nodeBounds = new RectangleF(x, y, 100, 60);
            ShapeNode node = diagram.Factory.CreateShapeNode(nodeBounds);
            node.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(100, 206, 0, 0));            
            node.Font = new Font("Arial", 26);
            node.TextBrush = new MindFusion.Drawing.SolidBrush(Color.Black);
            node.Visible = false;
            node.Shape = Shapes.RoundRect;
            node.Id = "Occupied";

        }

        private void loadEvents()
        {         

            List<Performance> performances = records.Performances.ToList();     

            foreach (Performance p in performances)
            {
                Appointment app = new Appointment();
                app.StartTime = (DateTime)p.Date;
                app.EndTime = app.StartTime.AddHours(2.0);           
                app.HeaderText = "";
                app.DescriptionText = p.Event.Title;
                app.Tag = p.Event.EventType_Id;
                app.Style.Brush = getBrush((int)app.Tag);
                calendar.Schedule.Items.Add(app);

            }

        }


        private MindFusion.Drawing.Brush getBrush( int eventType_id)
        {
            switch(eventType_id)
            {
                //ballet
                case 0:
                    return new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));
                //concert
                case 1:
                    return new MindFusion.Drawing.SolidBrush(Color.FromArgb(90, 121, 165));
                //opera
                default:
                    return new MindFusion.Drawing.SolidBrush(Color.FromArgb(0, 0, 99));
            }
        }


        private void Calendar_TooltipDisplaying(object sender, MindFusion.Scheduling.WinForms.TooltipEventArgs e)
        {
            if(e.Element.GetType() != typeof(Appointment))
            {
                e.Tooltip = "";
            }
            
        }

        void CreateSection(int rowStart, int[] seatsPerRow,
            float topY, float seatWidth, float seatHeight,
            float xPadding, float yPadding, float arcRadius)
        {
            float y = topY;
            for (int r = seatsPerRow.Length - 1; r >= 0; r--)
            {
                var c = new PointF(0, y + arcRadius);
                int seats = seatsPerRow[r];
                float rowWidth = seats * seatWidth + (seats - 1) * xPadding;
                float x = -rowWidth / 2;
                for (int s = 0; s < seats; s++)
                {
                    double adjustedY = y;
                    if (arcRadius > 0)
                    {
                        float angle = 0;
                        float radius = 0;
                        MindFusion.Geometry.Geometry2D.Convert.CartesianToPolar(
                            c, new PointF(x, y), ref angle, ref radius);
                        adjustedY = c.Y - arcRadius * Math.Sin(angle * Math.PI / 180);
                    }
                    ShapeNode seat = diagram.Factory.CreateShapeNode(
                        x, (float)adjustedY, seatWidth, seatHeight);
                    seat.Id = new Seat(rowStart + r, s);
                    seat.Tag = calcSeatCoeff((Seat)seat.Id);
                    seat.Font = new Font("Arial", 24);
                    x += seatWidth + xPadding;
                }
                y += seatHeight + yPadding;
            }
        }

     
        //retrieve all occupied seats for this performance
        private void calendar_ItemClick(object sender, MindFusion.Scheduling.WinForms.ItemMouseEventArgs e)
        {           

            IQueryable<Performance> _performances = records.Performances.AsQueryable();
            IQueryable<TICKETS_SOLD> _allTicketsSold = records.TICKETS_SOLD.AsQueryable();

            Appointment app = e.Item as Appointment;

            if (app != null)
            {
                var _tPerformances = _performances.Where(p => p.Date == app.StartTime);

                foreach (var _tPerformance in _tPerformances)
                {
                    var _ticketsSold = _allTicketsSold.Where(p => p.Performance_Id == _tPerformance.Id);
                    calendar.Tag = _tPerformance.Id;
                    title.Text = app.DescriptionText;

                    updateSeats((double)_tPerformance.BasePrice);

                    foreach (var occupiedSeat in _ticketsSold)
                    {
                        Seat seat = new Seat (occupiedSeat.Row_Index, 
                            occupiedSeat.Seat_Index);

                        ShapeNode node = diagram.FindNodeById(seat) as ShapeNode;

                        if (node != null)
                            node.Text = "X";
                    }                    
                }

            }
        }

        
        //clears the X sign at an occupied seat
        private void updateSeats( double basePrice )
        {
            foreach( var seat in diagram.Nodes)
            {
                (seat as ShapeNode).Text = "";
                if (seat.Tag != null)
                {
                   seat.Brush = getSeatBrush(basePrice * (double)seat.Tag);
                }
            }
        }

        private void diagram_NodeClicked(object sender, NodeEventArgs e)
        {
            ShapeNode seat = e.Node as ShapeNode;
            if(seat != null)
            {
                if(seat.Text == "X")
                {
                    ShapeNode warning = (ShapeNode)diagram.FindNodeById("Occupied");

                    if(warning != null)
                    {
                        warning.Text = "This Seat is Occupied!";
                        Timer timer = new Timer();
                        timer.Interval = 2500;
                        timer.Tick += Timer_Tick;
                        warning.Visible = true;
                        timer.Start();
                    }
                }
                else
                {
                    if (calendar.Tag != null)
                    {
                        Seat seatInfo = (Seat)seat.Id;
                        TICKETS_SOLD _ticket = new TICKETS_SOLD() { };
                        _ticket.Id = records.TICKETS_SOLD.ToList().Count;
                        _ticket.Performance_Id = (int)calendar.Tag;
                        _ticket.Row_Index = seatInfo.Row;
                        _ticket.Seat_Index = seatInfo.Place;
                        records.TICKETS_SOLD.Add(_ticket);
                        
                        try
                        {
                            records.SaveChanges();
                            List<TICKETS_SOLD> _allTicketsSold =
                                records.TICKETS_SOLD.AsEnumerable().ToList();
                            int i = _allTicketsSold.Count;

                        }
                        catch (DbEntityValidationException ex)
                        {
                            Debug.WriteLine(ex.Message);
                        }
                        seat.Text = "X";
                    }

                }
                    
            }
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            ShapeNode warning = (ShapeNode)diagram.FindNodeById("Occupied");
            warning.Visible = false;

        }

        /* calculate the price of a seat based on its location in the theatre hall */
        private double calcSeatCoeff(Seat seat)
        {
            if (seat.Row < 6)
                return 2;

            if (seat.Place > 3 && seat.Place < 10)
                return 1.5;

            return 1.0;
        }

        private MindFusion.Drawing.Brush getSeatBrush ( double price )
        {
            if (price >= 100)
                return new MindFusion.Drawing.SolidBrush(Color.FromArgb(97, 106, 127));

            if (price >= 80)
                return new MindFusion.Drawing.SolidBrush(Color.FromArgb(90, 121, 165));

            if (price >= 60)
                return new MindFusion.Drawing.SolidBrush(Color.FromArgb(102, 154, 204));

            if (price >= 40)
                return new MindFusion.Drawing.SolidBrush(Color.FromArgb(156, 170, 198));

            if (price > 20)
                return new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));
      

            return new MindFusion.Drawing.SolidBrush(Color.FromArgb(224, 233, 233));


        }



    }
}
